C ALGORITHM 565
C
C PDETWO/PSTEM/GEARB: SOLUTION OF SYSTEMS OF TWO DIMENSIONAL
C NONLINEAR PARTIAL DIFFERENTIAL EQUATIONS
C
C BY D.K. MELGAARD AND R.F. SINCOVEC
C
C ACM TRANSACTIONS ON MATHEMATICAL SOFTWARE 7,1 (MARCH 1981)
C
C
C     MAIN PROGRAM TO ILLUSTRATE THE USAGE OF THE PDETWO PACKAGE
C     USING EXAMPLES 1, 2, AND 3.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DOUBLE PRECISION  H,S,X,ERR1,DX,Y,TOUT,REALN1,DY,U1,HUSED,
     X                  EXP,R,T0,EPS,ERMAX,ABS,WORK,DL,DLI,DEV,
     X                  U2,U3,ERR2,ERR3,REALN2,REALN3
      INTEGER IX,NPDE,NSTEP,NX,NFE,NODE,MF,NY,
     X        NJE,NQUSED,IY,INDEX,I,IWORK,KODE,IK
     X        MORDER,LPW,LU
      COMMON /GEAR3/ HUSED,NQUSED,NSTEP,NFE,NJE
      COMMON /PROB/ DL,DLI,KODE
      INTEGER NRWK,NRIWK,NX_MAX,NY_MAX
C      PARAMETER( NRWK=97122838, NRIWK=2000000 )
C      PARAMETER( NX_MAX=2000, NY_MAX=2000 )
      DIMENSION U1(10,10),ERR1(10,10),REALN1(10,10)
      DIMENSION U2(5,31),ERR2(5,31),REALN2(5,31)

C      DIMENSION U3(2,5,5),ERR3(2,5,5),REALN3(2,5,5)
      DIMENSION U3(2,50,50),ERR3(2,50,50),REALN3(2,50,50)

C      DIMENSION U3(2,NX_MAX,NY_MAX),ERR3(2,NX_MAX,NY_MAX)
C      DIMENSION REALN3(2,NX_MAX,NY_MAX)
C      DIMENSION WORK(NRWK),IWORK(NRIWK),X(NX_MAX),Y(NY_MAX)

C      DIMENSION WORK(4186),IWORK(155),X(10),Y(31)
      DIMENSION WORK(1570738),IWORK(5000),X(50),Y(50)

      EQUIVALENCE (U1(1,1),U3(1,1,1)),(ERR1(1,1),ERR3(1,1,1)),
     X            (REALN1(1,1),REALN3(1,1,1))
      EQUIVALENCE (U2(1,1),U3(1,1,1)),(ERR2(1,1),ERR3(1,1,1)),
     X            (REALN2(1,1),REALN3(1,1,1))
C
      DO 1000 IK=1,3
      KODE = IK
      IF(KODE.EQ.1)  GO TO 100
      IF(KODE.EQ.2)  GO TO 300
      IF(KODE.EQ.3)  GO TO 500
 100  CONTINUE
C***********************************************************************
C
C  EXAMPLE 1.  ELLIPTIC PDE
C
C***********************************************************************
      WRITE (6,110)
  110 FORMAT(1H1,3X,36HOUTPUT FROM EXAMPLE 1.  ELLIPTIC PDE//)
C
C  DEFINE THE PROBLEM PARAMETERS.
C
      NX=10
      NY=10
      NPDE=1
      NODE=NPDE*NX*NY
      MF=22
      INDEX=1
      T0=0.0D+00
      H=0.1D-06
      EPS=0.1D-06
      DX = 1.0D+00/(DFLOAT(NX)-1.0D+00)
      DY = 1.0D+00/(DFLOAT(NY)-1.0D+00)
      DO 120 IX=1,NX
        Y(IX)=DFLOAT(IX)*DY-DY
  120   X(IX)=DFLOAT(IX)*DX-DX
      IWORK(1) = NPDE
      IWORK(2) = NX
      IWORK(3) = NY
      IWORK(4) = 5
      IWORK(5) = 4186
      IWORK(6) = 100
C
C  CALCULATE THE ACTUAL SOLUTION
C
      DO 130 IY = 1,NY
        DO 130 IX = 1,NX
          R = X(IX)
          S = Y(IY)
  130     REALN1(IX,IY)=3.0D+00*DEXP(R)*DEXP(S)*(R-R*R)*(S-S*S)
      WRITE (6,140) ((REALN1(IX,IY),IX=1,NX),IY=1,NY)
  140 FORMAT (/23H STEADY STATE SOLUTION /10(/3H    ,10E11.4))
C
C  DEFINE THE INITIAL CONDITION
C
      DO 160 IY = 1,NY
        DO 160 IX = 1,NX
  160    U1(IX,IY)=0.0D+00
      WRITE (6,170) NODE,T0,H,EPS,MF,U1
  170 FORMAT (//6H NODE ,I3,4H T0 ,F9.2,3H H ,E8.1,5H EPS ,E8.1,4H MF ,
     X        I2 //16H INITIAL VALUES / 10(/3H    ,10E11.2))
C
C  SET UP THE LOOP FOR CALLING THE INTEGRATOR AT DIFFERENT TOUT VALUES
C
      TOUT=.001D+00
      DO 260 I=1,5
      WRITE (6,200) TOUT
  200 FORMAT (// 5H TOUT,E15.6)
C
C  CALL THE INTEGRATOR
C
      write(*,*) 'Before DRIVEP'
      CALL DRIVEP (NODE,T0,H,U1,TOUT,EPS,MF,INDEX,WORK,IWORK,X,Y)
      write(*,*) 'INDEX=',INDEX
C
C  CHECK ERROR RETURN
C
      IF (INDEX .NE. 0) GO TO 1000
C
C  OUTPUT THE RESULTS
C
      WRITE (6,220) HUSED,NQUSED,NSTEP,NFE,NJE
  220 FORMAT (/7H HUSED ,E10.4,7H ORDER ,I3,7H NSTEP ,I6,5H NFE ,I5,
     X        5H NJE ,I5)
      WRITE (6,230) ((U1(IX,IY),IX=1,NX),IY=1,NY)
  230 FORMAT ( /9H U VALUES  //(3H    ,10E11.4))
C
C  DETERMINE THE ERROR
C
      ERMAX = 0.0D+00
      DO 240 IY = 1,NY
         DO 240 IX = 1,NX
          ERR1(IX,IY) = U1(IX,IY) - REALN1(IX,IY)
          IF (DABS(ERMAX) .LT. DABS(ERR1(IX,IY))) ERMAX=ERR1(IX,IY)
  240     CONTINUE
      WRITE (6,250) ERMAX
  250 FORMAT (/51H MAXIMUM DIFFERENCE FROM THE STEADY STATE SOLUTION //
     X        (3H    ,E11.4))
        TOUT=TOUT*10.D+00
  260 CONTINUE
C
C************************** END OF EXAMPLE 1. **************************
C
      GO TO 1000
 300  CONTINUE
C***********************************************************************
C
C  EXAMPLE 2.  BURGERS EQUATION
C
C***********************************************************************
      WRITE (6,310)
  310 FORMAT(1H1,3X,40HOUTPUT FROM EXAMPLE 2.  BURGERS EQUATION//)
C
C  DEFINE THE PROBLEM PARAMETERS.
C
      DL=.010D+00
      DLI=1.0D+00/(2.0D+00*DL)
      NX=5
      NY=31
      NPDE=1
      NODE=NX*NY*NPDE
      MF=10
      INDEX=1
      T0=0.0D+00
      H=0.1D-06
      EPS=0.1D-03
      DY = 1.0D+00/(DFLOAT(NY)-1.0D+00)
      DX = DY
      DO 320 I=1,NX
  320   X(I)=DFLOAT(I)*DX-DX
      DO 330 I=1,NY
  330   Y(I)=DFLOAT(I)*DY-DY
      IWORK(1) = NPDE
      IWORK(2) = NX
      IWORK(3) = NY
      IWORK(4) = 12
      IWORK(5) = 2687
      IWORK(6) = 155
C
C  DEFINE THE INITIAL CONDITION
C
      DO 360 IY = 1,NY
        DO 360 IX = 1,NX
          DEV=DEXP((-X(IX)-Y(IY)+T0)*DLI)
  360     U2(IX,IY)=DEV/(1.0D+00+DEV)
      WRITE (6,370) NODE,T0,H,EPS,MF,((U2(IX,IY),IX=1,NX),IY=1,NY)
  370 FORMAT (6H NODE ,I3,4H T0 ,F9.2,3H H ,E8.1,5H EPS ,E8.1,4H MF ,I2
     X        //16H INITIAL VALUES / 31(/3H    , 5E11.4))
C
C  SET UP THE LOOP FOR CALLING THE INTEGRATOR AT DIFFERENT TOUT VALUES
C
      TOUT=0.0D+00
      DO 460 I=1,4
      TOUT=TOUT+.10D+00
      WRITE (6,400) TOUT
  400 FORMAT (// 5H TOUT,E15.6)
C
C  CALCULATE THE ACTUAL SOLUTION
C
      DO 410 IY = 1,NY
        DO 410 IX = 1,NX
          DEV=DEXP((-X(IX)-Y(IY)+TOUT)*DLI)
  410     REALN2(IX,IY)=DEV/(1.0D+00+DEV)
C
C  CALL THE INTEGRATOR
C
      CALL DRIVEP (NODE,T0,H,U2,TOUT,EPS,MF,INDEX,WORK,IWORK,X,Y)
C
C  CHECK ERROR RETURN
C
      IF (INDEX .NE. 0) GO TO 1000
C
C  OUTPUT THE RESULTS
C
      WRITE (6,420) HUSED,NQUSED,NSTEP,NFE,NJE
  420 FORMAT (/7H HUSED ,E10.4,7H ORDER ,I3,7H NSTEP ,I6,5H NFE ,I5,
     X        5H NJE ,I5)
      WRITE (6,430) ((U2(IX,IY),IX=1,NX),IY=1,NY)
  430 FORMAT ( /9H U VALUES  // (3H    ,5E11.4))
C
C  DETERMINE THE ERROR
C
      ERMAX=0.0D+00
      DO 440 IY = 1,NY
         DO 440 IX = 1,NX
          ERR2(IX,IY)=U2(IX,IY)-REALN2(IX,IY)
          IF (DABS(ERMAX) .LT. DABS(ERR2(IX,IY))) ERMAX=ERR2(IX,IY)
  440     CONTINUE
      WRITE (6,450) ERMAX
  450 FORMAT (/13H MAX ERROR U ,E11.4)
      WRITE (6,455) ((ERR2(IX,IY),IX=1,NX),IY=1,NY)
  455 FORMAT (/6H ERROR // (3H    ,5E11.4))
  460 CONTINUE
C
C************************** END OF EXAMPLE 2. **************************
C
      GO TO 1000
 500  CONTINUE
C***********************************************************************
C
C  EXAMPLE 3.  COUPLED SYSTEM OF PDE*S
C
C***********************************************************************
      WRITE (6,510)
  510 FORMAT(1H1,3X,47HOUTPUT FROM EXAMPLE 3.  COUPLED SYSTEM OF PDE*S//
     X)
C
C  DEFINE THE PROBLEM PARAMETERS.  NPDE,NX, AND NY PRIMARILY DETERMINE
C  THE DIMENSIONS FOR THE ARRAYS IN PDETWO AND THE MODIFIED GEARB.
C
C      NX=5
C      NY=5
      NX=50
      NY=50
      NPDE=2
      NODE=NX*NY*NPDE
C     MF=10*METH+MITER
      MF=22
C     METH=2; MITER=2
      INDEX=1
      T0=0.0D+00
      H=0.1D-06
C     This local accuracy requirement seems to stringent and when relaxed
C     we converge much more nicely...
      EPS=0.1E-3
C      EPS=0.1D-2
      DX=1.0D+00/(DFLOAT(NX)-1.0D+00)
      DO 520 IX=1,NX
        X(IX)=DFLOAT(IX)*DX-DX
  520   Y(IX)=X(IX)
      IWORK(1) = NPDE
      IWORK(2) = NX
      IWORK(3) = NY

      MORDER = 5
      IWORK(4) = MORDER
      LPW = (3*(NX+1)*NPDE-2)*NODE
      LU  = NODE*(MORDER+1)
C      IWORK(5) = 2308
C      IWORK(6) = 50
      IWORK(5) = 1570738
      IWORK(6) = 5000
C      IWORK(5) = NPDE * ( NPDE*(NX*2+3) + 13 + NX ) + NX*4 + 
C     &           4*NODE + LPW + LU
C      IWORK(6) = NODE
C
C  DEFINE THE INITIAL CONDITIONS
C
      DO 560 IY=1,NY
        DO 560 IX=1,NX
          U3(1,IX,IY)=X(IX)+Y(IY)
  560     U3(2,IX,IY)=2.0D+00*X(IX)+3.0D+00*Y(IY)
      WRITE (6,570) NODE,T0,H,EPS,MF,((U3(1,IX,IY),IX=1,NX),IY=1,NY),
     X             ((U3(2,IX,IY),IX=1,NX),IY=1,NY)
  570 FORMAT (6H NODE ,I3,4H T0 ,F9.2,3H H ,E8.1,5H EPS ,E8.1,4H MF ,I2
     X        //19H INITIAL U1 VALUES  / 5(/3H    ,5E11.4)
     X        //19H INITIAL U2 VALUES  / 5(/3H    ,5E11.4))
C
C  SET UP THE LOOP FOR CALLING THE INTEGRATOR AT DIFFERENT TOUT VALUES
C
      TOUT=.25D+00
      DO 660 I=1,4
      WRITE (6,600) TOUT
  600 FORMAT (//5H TOUT ,E15.6)
C
C  CALCULATE THE ACTUAL SOLUTION
C
      DO 610 IY=1,NY
        DO 610 IX=1,NX
          REALN3(2,IX,IY)=TOUT+2.0D+00*X(IX)+3.0D+00*Y(IY)
  610     REALN3(1,IX,IY)=TOUT +X(IX)+Y(IY)
C
C  CALL THE INTEGRATOR
C
      CALL DRIVEP (NODE,T0,H,U3,TOUT,EPS,MF,INDEX,WORK,IWORK,X,Y)
C
C  CHECK ERROR RETURN
C
      IF (INDEX .NE. 0) GO TO 1000
C
C  OUTPUT THE RESULTS
C
      WRITE (6,620) HUSED,NQUSED,NSTEP,NFE,NJE
  620 FORMAT (/7H HUSED ,E10.4,7H ORDER ,I3,7H NSTEP ,I6,5H NFE ,I5,
     X        5H NJE ,I5)
      WRITE (6,630) ((U3(1,IX,IY),IX=1,NX),IY=1,NY)
  630 FORMAT (/11H U1 VALUES //(3H    ,5E11.4))
      WRITE (6,635) ((U3(2,IX,IY),IX=1,NX),IY=1,NY)
  635 FORMAT (/11H U2 VALUES //(3H    ,5E11.4))
C
C  DETERMINE THE ERROR
C
      DO 640 IY=1,NY
         DO 640 IX=1,NX
          ERR3(1,IX,IY)=U3(1,IX,IY)-REALN3(1,IX,IY)
          ERR3(2,IX,IY)=U3(2,IX,IY)-REALN3(2,IX,IY)
  640     CONTINUE
      WRITE (6,650) ((ERR3(1,IX,IY),IX=1,NX),IY=1,NY)
  650 FORMAT (/9H ERROR U1 //(3H    ,5E11.4))
      WRITE (6,655) ((ERR3(2,IX,IY),IX=1,NX),IY=1,NY)
  655 FORMAT (/9H ERROR U2 //(3H    ,5E11.4))
        TOUT=TOUT+.25D+00
  660 CONTINUE
C
C************************** END OF EXAMPLE 3. **************************
C
 1000 CONTINUE
      STOP
      END
      SUBROUTINE BNDRYV (T,X,Y,U,AV,BV,CV,NPDE)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  DEFINE THE VERTICAL BOUNDARY CONDITIONS
C
      DOUBLE PRECISION  T,U,X,Y,BV,AV,CV
      INTEGER NPDE
      COMMON /PROB/ DL,DLI,KODE
      DIMENSION U(NPDE),AV(NPDE),BV(NPDE),CV(NPDE)
      IF(KODE.EQ.1)  GO TO 100
      IF(KODE.EQ.2)  GO TO 200
      IF(KODE.EQ.3)  GO TO 300
C
C  EXAMPLE 1.  ELLIPTIC PDE
C
  100 CONTINUE
      AV(1) = 1.0D+00
      BV(1) = 0.0D+00
      CV(1) = 0.0D+00
      GO TO 400
C
C  EXAMPLE 2.  BURGERS EQUATION
C
  200 CONTINUE
      AV(1) = 1.0D+00
      BV(1) = 0.0D+00
      DEV = DEXP((-X-Y+T)*DLI)
      CV(1) = DEV/(1.0D+00+DEV)
      GO TO 400
C
C  EXAMPLE 3.  COUPLED SYSTEM OF PDE*S
C
  300 CONTINUE
      IF (X .NE. 0.0D+00) GO TO 310
      AV(1)=1.0D+00
      BV(1)=0.0D+00
      CV(1)=T+Y
      AV(2)=1.0D+00
      BV(2)=0.0D+00
      CV(2)=T+3.0D+00*Y
      GO TO 400
  310 CONTINUE
      AV(1)=1.0D+00
      BV(1)=0.0D+00
      CV(1)=T+1.0D+00+Y
      AV(2)=0.0D+00
      BV(2)=1.0D+00
      CV(2)=2.0D+00
  400 CONTINUE
      RETURN
      END
      SUBROUTINE BNDRYH (T,X,Y,U,AH,BH,CH,NPDE)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  DEFINE THE HORIZONTAL BOUNDARY CONDITIONS
C
      DOUBLE PRECISION  T,U,X,Y,BH,AH,CH
      INTEGER NPDE
      COMMON /PROB/ DL,DLI,KODE
      DIMENSION U(NPDE),AH(NPDE),BH(NPDE),CH(NPDE)
      IF(KODE.EQ.1)  GO TO 100
      IF(KODE.EQ.2)  GO TO 200
      IF(KODE.EQ.3)  GO TO 300
C
C  EXAMPLE 1.  ELLIPTIC PDE
C
  100 CONTINUE
      AH(1) = 1.0D+00
      BH(1) = 0.0D+00
      CH(1) = 0.0D+00
      GO TO 400
C
C  EXAMPLE 2.  BURGERS EQUATION
C
  200 CONTINUE
      AH(1) = 1.0D+00
      BH(1) = 0.0D+00
      DEV = DEXP((-X-Y+T)*DLI)
      CH(1) = DEV/(1.0D+00+DEV)
      GO TO 400
C
C  EXAMPLE 3.  COUPLED SYSTEM OF PDE*S
C
  300 CONTINUE
      IF (Y .NE. 0.0D+00) GO TO 310
      AH(1)=0.0D+00
      BH(1)=1.0D+00
      CH(1)=1.0D+00
      AH(2)=1.0D+00
      BH(2)=1.0D+00
      CH(2)=T+2.0D+00*X+3.0D+00
      GO TO 400
  310 CONTINUE
      AH(1)=0.0D+00
      BH(1)=U(2)
      CH(1)=T+2.0D+00*X+3.0D+00
      AH(2)=1.0D+00
      BH(2)=0.0D+00
      CH(2)=T+2.0D+00*X+3.0D+00
  400 CONTINUE
      RETURN
      END
      SUBROUTINE DIFFH (T,X,Y,U,DH,NPDE)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  DEFINE THE HORIZONTAL DIFFUSION COEFFICIENTS
C
      DOUBLE PRECISION  T,U,X,Y,DH
      INTEGER NPDE
      COMMON /PROB/ DL,DLI,KODE
      DIMENSION U(NPDE),DH(NPDE,NPDE)
      IF(KODE.EQ.1)  GO TO 100
      IF(KODE.EQ.2)  GO TO 200
      IF(KODE.EQ.3)  GO TO 300
C
C  EXAMPLE 1.  ELLIPTIC PDE
C
  100 CONTINUE
      DH(1,1) = 1.0D+00
      GO TO 400
C
C  EXAMPLE 2.  BURGERS EQUATION
C
  200 CONTINUE
      DH(1,1) = 1.0D+00
      GO TO 400
C
C  EXAMPLE 3.  COUPLED SYSTEM OF PDE*S
C
  300 CONTINUE
      DH(1,1)=0.0D+00
      DH(1,2)=0.0D+00
      DH(2,1)=U(2)
      DH(2,2)=U(1)
  400 CONTINUE
      RETURN
      END
      SUBROUTINE DIFFV (T,X,Y,U,DV,NPDE)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  DEFINE THE VERTICAL DIFFUSION COEFFICIENTS
C
      DOUBLE PRECISION  T,U,X,Y,DV
      INTEGER NPDE
      COMMON /PROB/ DL,DLI,KODE
      DIMENSION U(NPDE),DV(NPDE,NPDE)
      IF(KODE.EQ.1)  GO TO 100
      IF(KODE.EQ.2)  GO TO 200
      IF(KODE.EQ.3)  GO TO 300
C
C  EXAMPLE 1.  ELLIPTIC PDE
C
  100 CONTINUE
      DV(1,1) = 1.0D+00
      GO TO 400
C
C  EXAMPLE 2.  BURGERS EQUATION
C
  200 CONTINUE
      DV(1,1) = 1.0D+00
      GO TO 400
C
C  EXAMPLE 3.  COUPLED SYSTEM OF PDE*S
C
  300 CONTINUE
      DV(1,1)=U(2)
      DV(1,2)=U(1)
      DV(2,1)=0.0D+00
      DV(2,2)=0.0D+00
  400 CONTINUE
      RETURN
      END
      SUBROUTINE F(T,X,Y,U,UX,UY,DUXX,DUYY,DUDT,NPDE)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C  DEFINE THE PDE
C
      DOUBLE PRECISION  T,U,X,Y,UX,UY,DUXX,DUYY,DUDT,EXP
      INTEGER NPDE
      COMMON /PROB/ DL,DLI,KODE
      DIMENSION U(NPDE),UX(NPDE),UY(NPDE),DUXX(NPDE,NPDE),
     X DUYY(NPDE,NPDE),DUDT(NPDE)
      IF(KODE.EQ.1)  GO TO 100
      IF(KODE.EQ.2)  GO TO 200
      IF(KODE.EQ.3)  GO TO 300
C
C  EXAMPLE 1.  ELLIPTIC PDE
C
  100 CONTINUE
      DUDT(1) = DUXX(1,1) + DUYY(1,1) - 6.0D+00*X*Y*DEXP(X)*DEXP(Y)*
     X       (X*Y+X+Y-3.0D+00)
      GO TO 400
C
C  EXAMPLE 2.  BURGERS EQUATION
C
  200 CONTINUE
      DUDT(1) = -U(1)*(UX(1)+UY(1))+DL*(DUXX(1,1)+DUYY(1,1))
      GO TO 400
C
C  EXAMPLE 3.  COUPLED SYSTEM OF PDE*S
C
  300 CONTINUE
      DUDT(1)=U(2)*(DUYY(1,1)+DUYY(1,2))-3.0D+00*U(2)*UX(2)+4.0D+00*UY(1
     X)-UY(2)
      DUDT(2)=DUXX(2,1)+DUXX(2,2)-UY(2)
  400 CONTINUE
      RETURN
      END
